% Script for matched-phase adiabatic SLR code

% Some functions use John Pauly's RF design tools:
% http://rsl.stanford.edu/research/software.html
% Pauly J, Le Roux P, Nishimra D, Macovski A. 
% Parameter relations for the Shinnar-Le Roux selective excitation pulse design algorithm. 
% IEEE Tr Medical Imaging 1991; 10(1):53-65

% Designed by Hadrien Dyvorne & Priti Balchandani
% Mount Sinai School of Medicine, 2015
% hdyvorne@gmail.com

% If you use this code, please cite:

% H Dyvorne, R O'Halloran, P Balchandani
% Ultrahigh field single-refocused diffusion weighted imaging using a matched-phase adiabatic spin echo (MASE)
% Magn Reson Med, 2015

% Balchandani P, Khalighi MM, Glover G, Pauly J, Spielman D. 
% Selfrefocused adiabatic pulse for spin echo imaging at 7 T. 
% Magn Reson Med 2012;67:1077?1085.


%% FILTER DESIGN
clear
waveform = firls(355,[0 .007 .02 1],[1 1 0 0],[0.1/8,sqrt(0.01/2)]);
waveform = zeropad(waveform,4000);

%% ADD QUADRATIC PHASE TO SLICE PROFILE
slice = fftshift(fft(ifftshift(waveform)));
quadforce = 2.8e4;
w = abs(linspace(-1,1,length(slice)));
slice = abs(slice) .* exp(1i*quadforce*w.^2);

% SLR
% beta polynomial of quadratic profile
bz_180 = fftshift(ifft(ifftshift(slice)));
% Normalize so the max magnetization is 1
bz_180 = bz_180 / max(abs(fft(bz_180)));
% Truncate the beta at 5% (normally we would truncate the RF, 
% but designing a truncated matched 90 imposes to truncate the beta)
threshold = 0.05;
bz_norm = bz_180 / max(abs(bz_180));
bz_180 = bz_180(min(find(abs(bz_norm)>threshold)):max(find(abs(bz_norm)>threshold)));
% Derive alpha, then RF
az_180 = b2a_dual(bz_180);
rf_180 = ab2rf(az_180,bz_180);
% Scale to get waveform in Gauss
T = 4e-3; % duration
rf_180_scaled = rfscaleg(rf_180,T*1e3);
figure,subplot(2,1,1); plot(abs(rf_180_scaled)); subplot(2,1,2); plot(unwrap(angle(rf_180_scaled)))

%% SIMULATIONS FOR INVERSION OR REFOCUSING PULSE
x = -10:0.05:10;
g_180 = 2*pi*ones(size(rf_180))/length(rf_180);
[az180_slc,bz180_slc] = abrm(1.0*rf_180,g_180,x);
mz_refoc = bz180_slc.^2;

figure,
subplot(2,1,1); plot(x/T,abs(mz_refoc)); title('Magnitude Refocusing'); 
subplot(2,1,2); plot(x/T,unwrap(angle(mz_refoc))); title('Phase Refocusing');


%% BUILDING A MATCHED 90
bz_90 = bz_180;
bz_90=[zeros(1,length(bz_90)) bz_90];
bz_90=fft(ifft(bz_90).^2);
% 
bz_90=bz_90/sum(max(abs(fft(bz_90))));
bz_90=bz_90*sin(pi/4); % scaled to get a flip angle of 90

az_90 = b2a_dual(bz_90);
rf_90 = ab2rf(az_90,bz_90);

%% SIMULATION of a matched spin echo
g_90 = 2*2*pi*ones(size(rf_90))/length(rf_90);
[az90_slc,bz90_slc] = abrm(rf_90,g_90,x);
mxy = 2*conj(az90_slc).*bz90_slc;
mz_spinecho = conj(mxy) .* mz_refoc;

% Plot spin echo magnitude and phase
% corrected for linear phase
figure,
subplot(2,1,1); plot(abs(mz_spinecho)); title('Magnitude Matched spin echo'); 
subplot(2,1,2); plot(unwrap(angle(mz_spinecho.*exp(1i*2*pi*x')))); title('Phase Matched spin echo'); 

%% OVERDRIVE BEHAVIOR of the refocusing pulse

for ii=1:20
    [az180_slc,bz180_slc] = abrm((0.5+ii/10)*rf_180,g_180,x);
    mz_overdrive(:,ii) = bz180_slc.^2;
end
figure,imagesc(0.6:0.1:2.5,x/T,abs(mz_overdrive)); set(gca,'FontSize',24)


